#include"filetransdlg.h"

#include<qlabel.h>
#include<qlineedit.h>
#include<qpushbutton.h>
#include<qtimer.h>
#include<qfiledialog.h>
#include<qfile.h>
#include<qprogressbar.h>
#include<qdir.h>
#include"psiaccount.h"
#include"userlist.h"
#include"common.h"
#include"iconwidget.h"
#include"busywidget.h"
#include"filetransfer.h"

using namespace XMPP;

static QString lastPath;

class FileTransDlg::Private
{
public:
	PsiAccount *pa;
	Jid jid;
	QLabel *lb_ident;
	FileTransfer *ft;
	QFile f;
	QString fileName;
	int fileSize;
	int sent;
	bool sending;
};

FileTransDlg::FileTransDlg(const Jid &jid, PsiAccount *pa)
:FileTransUI(0, 0, false, WDestructiveClose)
{
	d = new Private;
	d->pa = pa;
	d->jid = jid;
	d->ft = 0;
	d->sending = true;

	d->lb_ident = d->pa->accountLabel(this, true);
	d->lb_ident->setSizePolicy(QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum ));
	replaceWidget(lb_accountlabel, d->lb_ident);

	UserListItem *u = d->pa->findFirstRelavent(d->jid);
	setCaption(tr("Send File: %1").arg(jidnick(u->jid().full(), u->name())));

	le_to->setText(u->jid().full());
	pb_start->setText(tr("&Send"));
	pb_stop->setText(tr("&Close"));

	connect(tb_browse, SIGNAL(clicked()), SLOT(chooseFile()));
	connect(pb_start, SIGNAL(clicked()), SLOT(doStart()));
	connect(pb_stop, SIGNAL(clicked()), SLOT(close()));

	pb_start->setFocus();
	d->pa->dialogRegister(this);
	QTimer::singleShot(0, this, SLOT(chooseFile()));
}

FileTransDlg::FileTransDlg(FileTransfer *ft, PsiAccount *pa)
:FileTransUI(0, 0, false, WDestructiveClose)
{
	d = new Private;
	d->pa = pa;
	d->jid = ft->peer();
	d->ft = ft;
	d->sending = false;

	d->fileName = ft->fileName();
	d->fileSize = ft->fileSize();

	d->lb_ident = d->pa->accountLabel(this, true);
	d->lb_ident->setSizePolicy(QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum ));
	replaceWidget(lb_accountlabel, d->lb_ident);

	UserListItem *u = d->pa->findFirstRelavent(d->jid);
	setCaption(tr("Receive File: %1").arg(jidnick(u->jid().full(), u->name())));

	QString fname = QDir::home().filePath(d->fileName);

	le_to->setText(u->jid().full());
	le_fname->setText(fname);
	pb_start->setText(tr("&Accept"));
	pb_stop->setText(tr("&Reject"));

	//connect(tb_browse, SIGNAL(clicked()), SLOT(chooseFile()));
	connect(pb_start, SIGNAL(clicked()), SLOT(doStart()));
	connect(pb_stop, SIGNAL(clicked()), SLOT(close()));

	pb_start->setFocus();
	d->pa->dialogRegister(this);
}

FileTransDlg::~FileTransDlg()
{
	delete d->ft;
	d->pa->dialogUnregister(this);
	delete d;
}

void FileTransDlg::done(int r)
{
	if(busy->isActive()) {
		int n = QMessageBox::information(this, tr("Warning"), tr("Are you sure you want to cancel the transfer?"), tr("&Yes"), tr("&No"));
		if(n != 0)
			return;
	}
	else {
		// if receiving and not started, then reject
		if(d->ft && !d->sending) {
			d->ft->reject();
			delete d->ft;
			d->ft = 0;
		}
	}
	QDialog::done(r);
}

void FileTransDlg::chooseFile()
{
	if(lastPath.isEmpty())
		lastPath = QDir::homeDirPath();
	QString str = QFileDialog::getOpenFileName(lastPath, tr("All files (*.*)"), this, 0, tr("Choose a file"));
	if(!str.isEmpty()) {
		QFileInfo fi(str);
		lastPath = fi.dirPath();
		le_fname->setText(fi.filePath());
	}
}

void FileTransDlg::doStart()
{
	if(d->sending) {
		d->f.setName(le_fname->text());
		if(!d->f.open(IO_ReadOnly)) {
			QMessageBox::information(this, tr("Error"), tr("Unable to open file for reading."));
			return;
		}

		pb_start->setEnabled(false);
		pb_stop->setText(tr("&Cancel"));
		pb_stop->setFocus();
		busy->start();

		QFileInfo fi(d->f.name());
		d->fileName = fi.fileName();
		d->fileSize = fi.size();

		d->ft = d->pa->client()->fileTransferManager()->createTransfer();
		connect(d->ft, SIGNAL(accepted()), SLOT(ft_accepted()));
		connect(d->ft, SIGNAL(connected()), SLOT(ft_connected()));
		connect(d->ft, SIGNAL(bytesWritten(int)), SLOT(ft_bytesWritten(int)));
		d->ft->sendFile(d->jid, d->fileName, d->fileSize);
	}
	else {
		pb_start->setEnabled(false);
		pb_stop->setText(tr("&Cancel"));
		pb_stop->setFocus();
		busy->start();

		connect(d->ft, SIGNAL(connected()), SLOT(ft_connected()));
		connect(d->ft, SIGNAL(readyRead(const QByteArray &)), SLOT(ft_readyRead(const QByteArray &)));
		d->ft->accept();
	}
}

void FileTransDlg::ft_accepted()
{
	printf("file transfer accepted\n");
}

void FileTransDlg::ft_connected()
{
	printf("file transfer connected!\n");
	d->sent = 0;
	progressBar->setTotalSteps(d->fileSize);

	if(d->sending) {
		trySend();
	}
	else {
		QFileInfo fi(le_fname->text());
		if(fi.exists()) {
			busy->stop();
			delete d->ft;
			d->ft = 0;
			QMessageBox::information(this, tr("Error"), tr("File already exists locally.  Cancelling transfer."));
			close();
			return;
		}
		d->f.setName(le_fname->text());
		if(!d->f.open(IO_WriteOnly | IO_Truncate)) {
			busy->stop();
			delete d->ft;
			d->ft = 0;
			QMessageBox::information(this, tr("Error"), tr("Unable to open file for writing."));
			close();
			return;
		}
	}
}

void FileTransDlg::ft_readyRead(const QByteArray &a)
{
	printf("%d bytes read\n", a.size());
	d->f.writeBlock(a.data(), a.size());
	d->sent += a.size();
	progressBar->setProgress(d->sent);
	if(d->sent == d->fileSize) {
		finished();
		return;
	}
}

void FileTransDlg::ft_bytesWritten(int x)
{
	printf("%d bytes written\n", x);
	d->sent += x;
	progressBar->setProgress(d->sent);
	if(d->sent == d->fileSize) {
		finished();
		return;
	}
	QTimer::singleShot(0, this, SLOT(trySend()));
}

void FileTransDlg::trySend()
{
	int blockSize = d->ft->dataSizeNeeded();
	QByteArray a(blockSize);
	int r = d->f.readBlock(a.data(), a.size());
	if(r < 0) {
		printf("error reading file data!\n");
		return;
	}
	if(r < (int)a.size())
		a.resize(r);
	d->ft->writeFileData(a);
}

void FileTransDlg::finished()
{
	printf("finished\n");
	d->f.close();
	delete d->ft;
	d->ft = 0;
	pb_stop->setText(tr("&Close"));
	busy->stop();

	closeDialogs(this);
}
